@@ -21,30 +21,32 @@
 #define GR_QNXGRAPHICS_H
 
 #include "xap_QNXApp.h"
-//#include "xap_QNXFont.h"
 #include "gr_Graphics.h"
 #include <string.h>
 #include <Pt.h>
+#include <photon/PhRender.h>
+#include "ut_vector.h"
 
 #include <ut_string.h>
 
 class UT_ByteBuf;
 
 class QNXFont : public GR_Font {
-        public:
-                QNXFont(const char *aFont) { 
-					m_fontstr = (aFont) ? UT_strdup(aFont) : NULL; 
-				};
-                ~QNXFont() { 
-					if (m_fontstr) { 
-						free(m_fontstr); 
-					} 
-				};
-                const char *getFont(void) { 
-					return(m_fontstr); 
-				};
-        private:
-                char   *m_fontstr;
+  public:
+		QNXFont(FontID *aFont); 
+		~QNXFont();
+		const int getDisplayFontSize(void);
+		const char *getDisplayFont(void);
+		void    createDisplayFont(UT_uint32 size);
+		void deleteDisplayFont();	const char *getFont(void); 
+	
+		const int getSize(void);
+		virtual UT_sint32 measureUnremappedCharForCache(UT_UCSChar cChar) const;
+
+	private:
+		FontID	*m_fontID;
+		FontID  *m_displayFontID;
+		FontID  *m_120ptFontID;
 };
 
 class GR_QNXGraphics : public GR_Graphics
@@ -53,11 +55,15 @@ class GR_QNXGraphics : public GR_Graphics
 	GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App * app);
 	~GR_QNXGraphics();
 
+	virtual void		drawGlyph(UT_uint32 glyph_idx,UT_sint32 xoff,UT_sint32 yoff);
 	virtual void 		drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void 		drawChars(const UT_UCSChar* pChars, int iCharOffset,
-						   		  int iLength, UT_sint32 xoff, UT_sint32 yoff);
+						   		  int iLength, UT_sint32 xoff, UT_sint32 yoff,
+								  int * pCharWidths = NULL);
 	virtual void 		setFont(GR_Font* pFont);
+//	virtual UT_uint32		measureString(const UT_UCSChar *s,int iOffset,int num,UT_GrowBufElement *pWidths);
 	virtual UT_uint32 	measureUnRemappedChar(const UT_UCSChar c);
+	virtual void 		getColor(UT_RGBColor& clr);
 	virtual void 		setColor(const UT_RGBColor& clr);
 
 	virtual GR_Font* 	getGUIFont();
@@ -70,12 +76,16 @@ class GR_QNXGraphics : public GR_Graphics
 	virtual UT_uint32 	getFontAscent();
 	virtual UT_uint32 	getFontDescent();
 	virtual UT_uint32 	getFontHeight();
+
+	virtual void getCoverage(UT_Vector &coverage);
 	virtual void 		drawLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void 		setLineWidth(UT_sint32);
 	virtual void 		xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
+	virtual void		polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints);
+	
 	virtual void 		polyLine(UT_Point * pts, UT_uint32 nPoints);
-	virtual void 		fillRect(UT_RGBColor& c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
-	virtual void 		fillRect(UT_RGBColor& c, UT_Rect &r);
+	virtual void 		fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
+	virtual void 		fillRect(const UT_RGBColor& c, UT_Rect &r);
 	virtual void 		invertRect(const UT_Rect* pRect);
 	virtual void 		setClipRect(const UT_Rect* pRect);
 	virtual void 		scroll(UT_sint32, UT_sint32);
@@ -113,21 +123,31 @@ class GR_QNXGraphics : public GR_Graphics
 	void 				init3dColors();
 	virtual void 		fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
 	virtual void 		fillRect(GR_Color3D c, UT_Rect &r);
+	virtual void setLineProperties ( double inWidthPixels, 
+					 GR_Graphics::JoinStyle inJoinStyle = JOIN_MITER,
+					 GR_Graphics::CapStyle inCapStyle   = CAP_BUTT,
+					 GR_Graphics::LineStyle inLineStyle = LINE_SOLID ) ;
 
 	/* GR_Font versions of the above information */
 	virtual UT_uint32 	getFontAscent(GR_Font *);
 	virtual UT_uint32 	getFontDescent(GR_Font *);
 	virtual UT_uint32 	getFontHeight(GR_Font *);
-
+	//Need this in the textdrawing cb.
+	PgColor_t getCurrentPgColor() { return m_currentColor; };
+	QNXFont *getCurrentQNXFont() { return m_pFont; };
+	PhGC_t	*getCurrentGC() { return m_pGC; };
  protected:
-	virtual UT_uint32 	_getResolution(void) const;
-	int 				DrawSetup();
-	int 				DrawTeardown();
+	virtual UT_uint32 	getDeviceResolution(void) const;
+	inline int 				DrawSetup();
+	inline int 				DrawTeardown();
 
 	PtWidget_t *  	m_pWin;
 	PtWidget_t *  	m_pDraw;
+	PdOffscreenContext_t *m_pOSC;
+	PhDrawContext_t				*m_pOldDC;
 	PhPoint_t		m_OffsetPoint;		
 	PhTile_t *		m_pClipList;		
+	struct _Pf_ctrl * m_pFontCx;	
 
 	// our currently requested font by handle
 	QNXFont *			m_pFont;
@@ -136,17 +156,26 @@ class GR_QNXGraphics : public GR_Graphics
   
 	void*  				m_pColormap;
 	UT_sint32			m_iLineWidth;
-	UT_sint32			m_iAscentCache, m_iDescentCache;
+	UT_sint32			m_iAscentCache, m_iDescentCache,m_iHeightCache;
 
 	GR_Graphics::Cursor		m_cursor;
 	GR_Graphics::ColorSpace	m_cs;
 	
 	PgColor_t			m_currentColor;
 	PgColor_t			m_3dColors[COUNT_3D_COLORS];
+	
+	PhGC_t				*m_pGC;
+	PhGC_t				*m_pGC_old;
 
 	UT_uint32			m_iShadowZoomPercentage;
  	bool             m_bPrintNextPage;    
 	PpPrintContext_t *  m_pPrintContext;
+private:
+	virtual void saveRectangle(UT_Rect &r, UT_uint32 iIndx);
+	virtual void restoreRectangle(UT_uint32 iIndx);
+
+	UT_Vector				m_vSaveRect;
+	UT_Vector 				m_vSaveRectBuf;
 };
 
 #endif /* GR_QNXGRAPHICS_H */
