@@ -1,5 +1,7 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Program Utilities
- * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,8 +28,6 @@
 #include "ut_vector.h"
 #include "ut_assert.h"
 
-#ifndef ABI_OPT_STL
-
 UT_Vector::UT_Vector(UT_uint32 sizehint)
   : m_pEntries(NULL), m_iCount(0), m_iSpace(0),
     m_iCutoffDouble(sizehint), m_iPostCutoffIncrement(32)
@@ -87,7 +87,7 @@ UT_sint32 UT_Vector::grow(UT_uint32 ndx)
 		new_iSpace = ndx;
 	}
 
-	void ** new_pEntries = (void **)realloc(m_pEntries, new_iSpace * sizeof(void *));
+	void ** new_pEntries = static_cast<void **>(realloc(m_pEntries, new_iSpace * sizeof(void *)));
 	if (!new_pEntries) {
 		return -1;
 	}
@@ -124,7 +124,7 @@ UT_sint32 UT_Vector::insertItemAt(void* p, UT_uint32 ndx)
 	return 0;
 }
 
-UT_sint32 UT_Vector::addItem(void* p, UT_uint32 * pIndex)
+UT_sint32 UT_Vector::addItem(const void* p, UT_uint32 * pIndex)
 {
 	UT_sint32 err = addItem(p);
 	if (!err && pIndex)
@@ -132,7 +132,7 @@ UT_sint32 UT_Vector::addItem(void* p, UT_uint32 * pIndex)
 	return err;
 }
 
-UT_sint32 UT_Vector::addItem(void* p)
+UT_sint32 UT_Vector::addItem(const void* p)
 {
 	if ((m_iCount+1) > m_iSpace)
 	{
@@ -143,7 +143,7 @@ UT_sint32 UT_Vector::addItem(void* p)
 		}
 	}
 
-	m_pEntries[m_iCount++] = p;
+	m_pEntries[m_iCount++] = (void *)p;  /*** bad, cast away const so we can build again ***/
 
 	return 0;
 }
@@ -164,6 +164,9 @@ UT_sint32 UT_Vector::setNthItem(UT_uint32 ndx, void * pNew, void ** ppOld)
 {
 	const UT_uint32 old_iSpace = m_iSpace;
 
+	// skip realloc in cases where we are removing an entry, as its probably an error
+	UT_return_val_if_fail(!((ndx >= m_iSpace) && (pNew == NULL) && (ppOld == NULL)), -1)
+
 	if (ndx >= m_iSpace)
 	{
 		const UT_sint32 err = grow(ndx+1);
@@ -221,7 +224,7 @@ UT_sint32 UT_Vector::findItem(void* p) const
 	{
 		if (m_pEntries[i] == p)
 		{
-			return (UT_sint32) i;
+			return static_cast<UT_sint32>(i);
 		}
 	}
 
@@ -254,109 +257,182 @@ const void* UT_Vector::operator[](UT_uint32 i) const
 	return this->getNthItem(i);
 }
 
-#else /* ABI_OPT_STL */
 
-UT_Vector::UT_Vector(UT_uint32 sizehint)
+
+UT_NumberVector::UT_NumberVector (UT_uint32 sizehint, UT_uint32 baseincr) :
+	m_pEntries(0),
+	m_iCount(0),
+	m_iSpace(0),
+	m_iInitialSize(sizehint),
+	m_iIncrement(baseincr)
 {
-	//Ignore the sizehint
+	// 
 }
 
-void UT_Vector::clear()
+UT_NumberVector::UT_NumberVector (const UT_NumberVector & NV) :
+	m_pEntries(0),
+	m_iCount(0),
+	m_iSpace(0),
+	m_iInitialSize(NV.m_iInitialSize),
+	m_iIncrement(NV.m_iIncrement)
 {
-	m_STLVec.clear();
+	copy (NV);
 }
 
-UT_Vector::~UT_Vector()
+UT_NumberVector::~UT_NumberVector ()
 {
+	clear ();
 }
 
-UT_uint32 UT_Vector::getItemCount() const
+UT_NumberVector & UT_NumberVector::operator= (const UT_NumberVector & NV)
 {
-	return m_STLVec.size();
+	copy (NV);
+	return *this;
 }
 
-UT_sint32 UT_Vector::insertItemAt(void* p, UT_uint32 ndx)
+bool UT_NumberVector::copy (const UT_NumberVector & NV)
 {
-	m_STLVec.insert(m_STLVec.begin()+ndx, p);
-	return 0;
-}
+	clear (false);
 
-UT_sint32 UT_Vector::addItem(void* p, UT_uint32 * pIndex)
-{
-	UT_sint32 err = addItem(p);
-	if (!err && pIndex)
-		*pIndex = m_STLVec.size()-1;
-	return err;
-}
+	if (NV.m_iCount > m_iSpace)
+		if (grow (NV.m_iCount) == -1)
+			return false; // :-(
 
-UT_sint32 UT_Vector::addItem(void* p)
-{
-	m_STLVec.push_back(p);
-
-	return 0;
-}
-
-void* UT_Vector::getNthItem(UT_uint32 n) const
-{
-	return m_STLVec[n];
-}
-
-UT_sint32 UT_Vector::setNthItem(UT_uint32 ndx, void * pNew, void ** ppOld)
-{
-	if (ppOld)
-	{
-		*ppOld = m_STLVec[ndx];
-	}
-	
-	if (m_STLVec.size() <= ndx)
-		m_STLVec.resize (ndx+1);
-	m_STLVec[ndx] = pNew;
-		
-	return 0;
-}
-
-void* UT_Vector::getLastItem() const
-{
-	return m_STLVec.back();
-}
-
-void* UT_Vector::getFirstItem() const
-{
-	return m_STLVec.front();
-}
-
-void UT_Vector::deleteNthItem(UT_uint32 n)
-{
-	m_STLVec.erase(m_STLVec.begin()+n);
-}
-
-UT_sint32 UT_Vector::findItem(void* p) const
-{
-	for (UT_uint32 i=0; i<m_STLVec.size(); i++)
-	{
-		if (m_STLVec[i] == p)
+	if (NV.m_iCount)
 		{
-			return (UT_sint32) i;
+			m_iCount = NV.m_iCount;
+			memcpy (m_pEntries, NV.m_pEntries, m_iCount * sizeof (UT_sint32));
 		}
-	}
-
-	return -1;
+	return true;
 }
 
-void UT_Vector::qsort(int (*compar)(const void *, const void *))
+void UT_NumberVector::clear (bool free_memory)
 {
-	sort(m_STLVec.begin(), m_STLVec.end(), compar);
+	if (free_memory)
+		{
+			FREEP(m_pEntries);
+			m_iSpace = 0;
+		}
+	m_iCount = 0;
 }
 
-bool UT_Vector::copy(UT_Vector *pVec)
+/* addItem() returns 0 on success, -1 on failure:
+ */
+UT_sint32 UT_NumberVector::addItem (UT_sint32 number)
 {
-	m_STLVec = pVec->m_STLVec;
+	if (grow (m_iCount + 1) == -1)
+		return -1; // :-(
+
+	m_pEntries[m_iCount++] = number;
+
 	return 0;
 }
 
-const void* UT_Vector::operator[](UT_uint32 i) const
+UT_sint32 UT_NumberVector::getNthItem (UT_uint32 index) const
 {
-	return m_STLVec[i];
+	UT_ASSERT(index < m_iCount);
+	return ((index < m_iCount) ? m_pEntries[index] : 0);
 }
 
-#endif /* ABI_OPT_STL */
+/* setNthItem() and insertItemAt() return 0 on success, -1 on failure:
+ */
+UT_sint32 UT_NumberVector::setNthItem (UT_uint32 index, UT_sint32 new_number, UT_sint32 * old_number)
+{
+	if (index >= m_iCount)
+		{
+			if (grow (index + 1) == -1)
+				return -1; // :-(
+
+			m_iCount = index + 1;
+		}
+
+	if (old_number) *old_number = m_pEntries[index];
+
+	m_pEntries[index] = new_number;
+
+	return 0;
+}
+
+UT_sint32 UT_NumberVector::insertItemAt (UT_sint32 number, UT_uint32 index)
+{
+	UT_ASSERT(index <= m_iCount);
+	if (index > m_iCount)
+		return -1;
+
+	if (grow (m_iCount + 1) == -1)
+		return -1; // :-(
+
+	if (index <  m_iCount)
+		memmove (m_pEntries + index + 1, m_pEntries + index, (m_iCount - index) * sizeof (UT_sint32));
+
+	++m_iCount;
+
+	m_pEntries[index] = number;
+
+	return 0;
+}
+
+void UT_NumberVector::deleteNthItem (UT_uint32 index)
+{
+	if (index >= m_iCount) return;
+
+	--m_iCount;
+
+	if (index <  m_iCount)
+		memmove (m_pEntries + index, m_pEntries + index + 1, (m_iCount - index) * sizeof (UT_sint32));
+
+	m_pEntries[m_iCount] = 0;
+}
+
+/* findItem() returns index >= 0 of first instance of number, or -1 if not found:
+ */
+UT_sint32 UT_NumberVector::findItem (UT_sint32 number) const
+{
+	UT_sint32 retval = -1;
+
+	for (UT_uint32 index = 0; index < m_iCount; index++)
+		if (m_pEntries[index] == number)
+			{
+				retval = static_cast<UT_sint32>(index);
+				break;
+			}
+	return retval;
+}
+
+/* grow() returns 0 on success, -1 on failure:
+ */
+UT_sint32 UT_NumberVector::grow (UT_uint32 requirement)
+{
+	if (requirement <= m_iSpace) return 0;
+
+	UT_uint32 new_space = m_iInitialSize;
+	if (new_space < requirement)
+		{
+			new_space = (requirement - m_iInitialSize) / m_iIncrement;
+			new_space = m_iInitialSize + new_space * m_iIncrement;
+
+			if (new_space < requirement)
+				new_space += m_iIncrement;
+		}
+	if (m_pEntries == 0)
+		{
+			m_pEntries = reinterpret_cast<UT_sint32 *>(malloc (new_space * sizeof (UT_sint32)));
+			UT_ASSERT(m_pEntries);
+			if (m_pEntries == 0)
+				return -1;
+			memset (m_pEntries, 0, new_space * sizeof (UT_sint32));
+		}
+	else
+		{
+			UT_sint32 * more = 0;
+			more = reinterpret_cast<UT_sint32 *>(realloc (m_pEntries, new_space * sizeof (UT_sint32)));
+			UT_ASSERT(more);
+			if (more == 0)
+				return -1;
+			m_pEntries = more;
+			memset (m_pEntries + m_iSpace, 0, (new_space - m_iSpace) * sizeof (UT_sint32));
+		}
+	m_iSpace = new_space;
+
+	return 0;
+}
