@@ -32,7 +32,7 @@
 #include "xap_App.h"
 #include "xap_Strings.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Win32FrameImpl.h"
 
 #include "xap_Dialog_Id.h"
 #include "xap_Dlg_FileOpenSaveAs.h"
@@ -41,9 +41,9 @@
 #include "gr_Win32Image.h"
 #include "gr_Win32Graphics.h"
 
-#include "..\..\..\wp\impexp\xp\ie_types.h"
-#include "..\..\..\wp\impexp\xp\ie_imp.h"
-#include "..\..\..\wp\impexp\xp\ie_impGraphic.h"
+#include "ie_types.h"
+#include "ie_imp.h"
+#include "ie_impGraphic.h"
 
 #include "xap_Win32Resources.rc2"
 
@@ -187,7 +187,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::_buildFilterList(UT_String& sFilter)
   user does not have a places bar capable common dialog box DLL
   anyway.
 */
-BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50* lpofn)
+BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50* lpofn, BOOL bSave)
 {
 		BOOL  bRslt;	
 		
@@ -196,8 +196,10 @@ BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50*
 		lpofn->dwReserved = 0;
 		lpofn->FlagsEx = 0;		
 		
-				
-		bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
+		if (bSave)				
+			bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
+		else
+			bRslt = GetOpenFileName((OPENFILENAME *)lpofn);
 		
 		if (!bRslt) // Error
 		{			
@@ -207,9 +209,12 @@ BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50*
 			{								
 					//  Try with the old one
 					lpofn->lStructSize = sizeof(OPENFILENAME);	
-					bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
-			}
+					if (bSave)				
+						bRslt = GetSaveFileName((OPENFILENAME *)lpofn);
+					else
+						bRslt = GetOpenFileName((OPENFILENAME *)lpofn);
 				
+			}				
 		}	
 		
 		return bRslt;
@@ -220,12 +225,12 @@ BOOL  XAP_Win32Dialog_FileOpenSaveAs::GetSaveFileName_Hooked(OPENFILENAME_WIN50*
 
 void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 {
+	UT_ASSERT(pFrame);
+	UT_ASSERT(pFrame->getFrameImpl());
+
 	XAP_Win32App* pWin32App = static_cast<XAP_Win32App*>(m_pApp);
 
-	m_pWin32Frame = static_cast<XAP_Win32Frame *>(pFrame);
-	UT_ASSERT(m_pWin32Frame);
-
-	HWND hFrame = m_pWin32Frame->getTopLevelWindow();
+	HWND hFrame = static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow();
 
 	char szFile[MAX_DLG_INS_PICT_STRING];	// buffer for filename
 	char szDir[MAX_DLG_INS_PICT_STRING];	// buffer for directory
@@ -358,7 +363,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLEHOOK;
 				
-		bDialogResult = GetSaveFileName_Hooked(&ofn);
+		bDialogResult = GetSaveFileName_Hooked(&ofn, TRUE);
 		break;
 
 	case XAP_DIALOG_ID_INSERT_PICTURE:
@@ -370,7 +375,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLETEMPLATE;
 		ofn.Flags |= OFN_ENABLEHOOK;
-		bDialogResult = GetSaveFileName_Hooked(&ofn);
+		bDialogResult = GetSaveFileName_Hooked(&ofn, FALSE);
 		break;
 
 	case XAP_DIALOG_ID_FILE_IMPORT:
@@ -386,7 +391,7 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		ofn.Flags |= OFN_OVERWRITEPROMPT;
 		ofn.Flags |= OFN_EXPLORER;
 		ofn.Flags |= OFN_ENABLEHOOK;
-		bDialogResult = GetSaveFileName_Hooked(&ofn);
+		bDialogResult = GetSaveFileName_Hooked(&ofn, TRUE);
 		break;
 
 	case XAP_DIALOG_ID_INSERT_FILE:
@@ -455,7 +460,6 @@ void XAP_Win32Dialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		UT_DEBUGMSG(("Didn't get a file: reason=0x%x\n", CommDlgExtendedError()));
 	}
 
-	m_pWin32Frame = NULL;
 	return;
 }
 
@@ -615,8 +619,8 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 	HWND hFrame 	= GetParent(hFOSADlg);
 	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
-	XAP_Win32Frame* pWin32Frame = (XAP_Win32Frame *) ( GetWindowLong(hFrame,GWL_USERDATA) );
-	const XAP_StringSet*  pSS	= pWin32Frame->getApp()->getStringSet();
+	const XAP_StringSet*  pSS	= XAP_App::getApp()->getStringSet();
+	UT_return_val_if_fail(pSS, false);
 
 	// Check if File Name is for a file
 	char buf[MAX_DLG_INS_PICT_STRING];
@@ -627,6 +631,14 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 		return false;
 	}
 
+	// Pass only files that can be openned
+	FILE* fitxer = fopen (buf,"r");
+
+	if (fitxer)
+		fclose(fitxer);
+	else
+		return false;
+
 	UT_DEBUGMSG(("File Selected is %s\n", buf));
 
 	// Load File into memory
@@ -721,10 +733,10 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_previewPicture(HWND hDlg)
 	PAINTSTRUCT ps;
 	HDC hdc = BeginPaint(hThumbnail, &ps);
 	FillRect(hdc, &r, GetSysColorBrush(COLOR_BTNFACE));
-	GR_Win32Graphics* pGr = new GR_Win32Graphics(hdc,hThumbnail,pWin32Frame->getApp());
+	GR_Win32Graphics* pGr = new GR_Win32Graphics(hdc,hThumbnail,XAP_App::getApp());
 	pGr->drawImage(pImage,
-				  (r.right	- scaled_width ) / 2,
-				  (r.bottom - scaled_height) / 2);
+				   pGr->tlu((r.right	- scaled_width ) / 2),
+				   pGr->tlu((r.bottom - scaled_height) / 2));
 	EndPaint(hThumbnail,&ps);
 
 	DELETEP(pBB);
@@ -740,9 +752,8 @@ UINT XAP_Win32Dialog_FileOpenSaveAs::_initPreviewDlg(HWND hDlg)
 	HWND hFrame 	= GetParent(hFOSADlg);
 	HWND hThumbnail = GetDlgItem(hDlg,XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW);
 
-	XAP_Win32Frame* pWin32Frame = (XAP_Win32Frame *) ( GetWindowLong(hFrame,GWL_USERDATA) );
-	XAP_App*			  pApp		  = pWin32Frame->getApp();
-	const XAP_StringSet*  pSS		  = pApp->getStringSet();
+	const XAP_StringSet*  pSS		  = XAP_App::getApp()->getStringSet();
+	UT_return_val_if_fail(pSS, false);
 	
 	SetDlgItemText( hDlg,
 					XAP_RID_DIALOG_INSERT_PICTURE_IMAGE_PREVIEW,
