@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -42,16 +42,70 @@
 bool pt_PieceTable::_unlinkStrux(pf_Frag_Strux * pfs,
 									pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
-	switch (pfs->getStruxType())	
+#if 1
+	if(pfs->getStruxType() == PTX_SectionTable)
+	{
+		UT_DEBUGMSG(("_unlink Strux Table %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_SectionCell)
+	{
+		UT_DEBUGMSG(("_unlink Strux Cell %x \n",pfs));
+	}	
+	else if(pfs->getStruxType() == PTX_EndTable)
+	{
+		UT_DEBUGMSG(("_unlink Strux End Table %x \n",pfs));
+	}	
+	else if(pfs->getStruxType() == PTX_EndCell)
+	{
+		UT_DEBUGMSG(("_unlink Strux EndCell %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_Block)
+	{
+		UT_DEBUGMSG(("_unlink Strux Block %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_Section)
+	{
+		UT_DEBUGMSG(("_unlink Strux Section %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_SectionHdrFtr)
+	{
+		UT_DEBUGMSG(("_unlink HdrFtr Strux Section %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_SectionFootnote)
+	{
+		UT_DEBUGMSG(("_unlink Strux SectionFootnote %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_EndFootnote)
+	{
+		UT_DEBUGMSG(("_unlink Strux EndFootnote %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_SectionEndnote)
+	{
+		UT_DEBUGMSG(("_unlink Strux SectionEndnote %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_EndEndnote)
+	{
+		UT_DEBUGMSG(("_unlink Strux EndEndnote %x \n",pfs));
+	}
+	m_pDocument->miniDump((PL_StruxDocHandle) pfs, 2);
+#endif
+	switch (pfs->getStruxType())
 	{
 	case PTX_Section:
 	case PTX_SectionHdrFtr:
 	case PTX_SectionEndnote:
+	case PTX_SectionTable:
+	case PTX_SectionCell:
+	case PTX_SectionFootnote:
+	case PTX_EndCell:
+	case PTX_EndTable:
+	case PTX_EndFootnote:
+	case PTX_EndEndnote:
 		return _unlinkStrux_Section(pfs,ppfEnd,pfragOffsetEnd);
-		
+
 	case PTX_Block:
 		return _unlinkStrux_Block(pfs,ppfEnd,pfragOffsetEnd);
-		
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		return false;
@@ -62,7 +116,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 										  pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
 	UT_ASSERT(pfs->getStruxType()==PTX_Block);
-	
+
 	// unlink this Block strux from the document.
 	// the caller is responsible for deleting pfs.
 
@@ -70,17 +124,11 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		*ppfEnd = pfs->getNext();
 	if (pfragOffsetEnd)
 		*pfragOffsetEnd = 0;
-	
+
 	// find the previous strux (either a paragraph or something else).
 
 	pf_Frag_Strux * pfsPrev = NULL;
-	pf_Frag * pf = pfs->getPrev();
-	while (pf && !pfsPrev)
-	{
-		if (pf->getType() == pf_Frag::PFT_Strux)
-			pfsPrev = static_cast<pf_Frag_Strux *> (pf);
-		pf = pf->getPrev();
-	}
+	_getStruxFromPosition(pfs->getPos(),&pfsPrev, true); // should that really skip footnotes?
 	UT_ASSERT(pfsPrev);			// we have a block that's not in a section ??
 	//
 	// Code to prevent a crash. But this should not happen and if it does not everything will
@@ -94,7 +142,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 
 	switch (pfsPrev->getStruxType())
 	{
-	
+
 	case PTX_Block:
 		// if there is a paragraph before us, we can delete this
 		// paragraph knowing that our content will be assimilated
@@ -102,8 +150,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
-	
-	case PTX_SectionEndnote:
+
 	case PTX_Section:
 		// we are the first paragraph in this section.  if we have
 		// content, we cannot be deleted, since there is no one to
@@ -132,10 +179,23 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		}
 
 		// no content in this paragraph.
-		
+
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
 
+
+	case PTX_SectionTable:
+	case PTX_SectionCell:
+	case PTX_EndCell:
+	case PTX_EndTable:
+//
+// deleting tables and cells is a mutlti-step process and we can make no assumptions
+// along the way.
+//
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+
 	default:
 		UT_ASSERT(0);
 		return false;
@@ -146,9 +206,17 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 											pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
 	UT_ASSERT(pfs->getStruxType()==PTX_Section
-	|| pfs->getStruxType()==PTX_SectionHdrFtr
-	|| pfs->getStruxType()==PTX_SectionEndnote);
-	
+			  || pfs->getStruxType()==PTX_SectionHdrFtr
+			  || pfs->getStruxType()==PTX_SectionEndnote
+			  || pfs->getStruxType()==PTX_SectionTable
+			  || pfs->getStruxType()==PTX_SectionCell
+			  || pfs->getStruxType()==PTX_EndCell
+			  || pfs->getStruxType()==PTX_EndTable 
+			  || pfs->getStruxType()==PTX_SectionFootnote 
+			  || pfs->getStruxType()==PTX_EndFootnote 
+			  || pfs->getStruxType()==PTX_SectionEndnote 
+			  || pfs->getStruxType()==PTX_EndEndnote );
+
 	// unlink this Section strux from the document.
 	// the caller is responsible for deleting pfs.
 
@@ -156,12 +224,12 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		*ppfEnd = pfs->getNext();
 	if (pfragOffsetEnd)
 		*pfragOffsetEnd = 0;
-	
+
 	// find the previous strux (either a paragraph or something else).
 
 	pf_Frag_Strux * pfsPrev = NULL;
 	pf_Frag * pf = pfs->getPrev();
-	while (pf && !pfsPrev)
+	while (pf && !pfsPrev || isFootnote(pf) || isEndFootnote(pf))
 	{
 		if (pf->getType() == pf_Frag::PFT_Strux)
 			pfsPrev = static_cast<pf_Frag_Strux *> (pf);
@@ -176,7 +244,7 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		UT_ASSERT(0);
 		return false;
 	}
-	
+
 	switch (pfsPrev->getStruxType())
 	{
 	case PTX_Block:
@@ -187,6 +255,67 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
+	case PTX_SectionTable:
+        //
+        // deleting tables is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_SectionCell:
+        //
+        // deleting tables is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_EndCell:
+        //
+        // deleting tables is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_EndTable:
+        //
+        // deleting tables is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_SectionEndnote:
+        //
+        // deleting Endnotes is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_EndEndnote:
+        //
+        // deleting Endnotes is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+
+	case PTX_SectionFootnote:
+        //
+        // deleting Footnotes is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_EndFootnote:
+        //
+        // deleting Footnotes is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
 
 	case PTX_Section:
 		// there are no blocks (paragraphs) between this section
@@ -202,7 +331,7 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		// and the previous section.  this is not possible.
 		// TODO decide if this should assert or just fail...
         //
-        // Actually this is OK if it's a hdrFtr that has not been 
+        // Actually this is OK if it's a hdrFtr that has not been
 		// "realized" yet. Like an even hdrftr that has been defined
         // but no even pages exist yet.
 		UT_DEBUGMSG(("No blocks between sections ??\n"));
@@ -215,10 +344,12 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		return false;
 	}
 }
-			
+
 bool pt_PieceTable::_deleteStruxWithNotify(PT_DocPosition dpos,
-											  pf_Frag_Strux * pfs,
-											  pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
+										   pf_Frag_Strux * pfs,
+										   pf_Frag ** ppfEnd,
+										   UT_uint32 * pfragOffsetEnd,
+										   bool bWithRec)
 {
 	PX_ChangeRecord_Strux * pcrs
 		= new PX_ChangeRecord_Strux(PX_ChangeRecord::PXT_DeleteStrux,
@@ -227,31 +358,10 @@ bool pt_PieceTable::_deleteStruxWithNotify(PT_DocPosition dpos,
 
 	if (!_unlinkStrux(pfs,ppfEnd,pfragOffsetEnd))
 		return false;
-	
+
 	// add record to history.  we do not attempt to coalesce these.
-	m_history.addChangeRecord(pcrs);
-	m_pDocument->notifyListeners(pfs,pcrs);
-
-	delete pfs;
-
-	return true;
-}
-
-			
-bool pt_PieceTable::_deleteStrux_norec(PT_DocPosition dpos,
-											  pf_Frag_Strux * pfs,
-											  pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
-{
-	PX_ChangeRecord_Strux * pcrs
-		= new PX_ChangeRecord_Strux(PX_ChangeRecord::PXT_DeleteStrux,
-									dpos, pfs->getIndexAP(), pfs->getStruxType());
-	UT_ASSERT(pcrs);
-
-	if (!_unlinkStrux(pfs,ppfEnd,pfragOffsetEnd))
-		return false;
-	
-	// No history for field updates..
-	// m_history.addChangeRecord(pcrs);
+	if (bWithRec)
+		m_history.addChangeRecord(pcrs);
 	m_pDocument->notifyListeners(pfs,pcrs);
 
 	delete pfs;
@@ -260,7 +370,7 @@ bool pt_PieceTable::_deleteStrux_norec(PT_DocPosition dpos,
 }
 
 /*!
- * This method scans the piecetAble from the section Frag_strux given looking 
+ * This method scans the piecetAble from the section Frag_strux given looking
  * for any Header/Footers that belong to the strux. If it finds them, they
  * are deleted with notifications.
 \params pf_Frag_Strux_Section pfStruxSec the Section strux that might have headers
@@ -271,7 +381,7 @@ bool pt_PieceTable::_deleteStrux_norec(PT_DocPosition dpos,
 bool pt_PieceTable::_deleteHdrFtrsFromSectionStruxIfPresent(pf_Frag_Strux_Section * pfStruxSec)
 {
 	//
-	// Get the index to the Attributes/properties of the section strux to see if 
+	// Get the index to the Attributes/properties of the section strux to see if
 	// if there is a header defined for this strux.
 	//
 	// FIXME: Handle all the new header/footer types.
@@ -288,56 +398,56 @@ bool pt_PieceTable::_deleteHdrFtrsFromSectionStruxIfPresent(pf_Frag_Strux_Sectio
 	{
 		HeaderV = szHeaderV;
 		vecHdrFtr.addItem((void *) HeaderV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("header-even",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		HeaderEvenV = szHeaderV;
 		vecHdrFtr.addItem((void *) HeaderEvenV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("header-last",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		HeaderLastV = szHeaderV;
 		vecHdrFtr.addItem((void *) HeaderLastV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("header-first",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		HeaderFirstV = szHeaderV;
 		vecHdrFtr.addItem((void *) HeaderFirstV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("footer",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		FooterV = szHeaderV;
 		vecHdrFtr.addItem((void *) FooterV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("footer-even",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		FooterEvenV = szHeaderV;
 		vecHdrFtr.addItem((void *) FooterEvenV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("footer-last",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		FooterLastV = szHeaderV;
 		vecHdrFtr.addItem((void *) FooterLastV.c_str());
-	} 
+	}
 	szHeaderV =  NULL;
 	bres = pAP->getAttribute("footer-first",szHeaderV);
 	if(szHeaderV && *szHeaderV && (UT_strcmp(szHeaderV,"0") != 0))
 	{
 		FooterFirstV = szHeaderV;
 		vecHdrFtr.addItem((void *) FooterFirstV.c_str());
-	} 
+	}
 	UT_sint32 countHdrFtr = (UT_sint32) vecHdrFtr.getItemCount();
 	UT_DEBUGMSG(("SEVIOR: Deleting HdrFtrs from Document, num Header/Footers %d\n",countHdrFtr));
 	if(0 == countHdrFtr)
@@ -345,7 +455,7 @@ bool pt_PieceTable::_deleteHdrFtrsFromSectionStruxIfPresent(pf_Frag_Strux_Sectio
 		return true;
 	}
 //
-// This section has a header or footer attribute. Scan the piecetable to see 
+// This section has a header or footer attribute. Scan the piecetable to see
 // if there is a header strux somewhere with an ID that matches our section.
 //
 	pf_Frag * curFrag = NULL;
@@ -413,10 +523,11 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 {
 //
 // First we need the document position of the header/footer strux.
-// 
+//
 	UT_DEBUGMSG(("SEVIOR: Deleting hdrftr \n"));
 	const pf_Frag * pfFrag = NULL;
 	pfFrag = static_cast<pf_Frag *>(pfFragStruxHdrFtr);
+	// TODO HdrFtrPos is unused
 	PT_DocPosition HdrFtrPos = getFragPosition(pfFrag);
 	UT_Vector vecFragStrux;
 	UT_DEBUGMSG(("SEVIOR: Deleting hdrftr Strux Pos = %d \n",HdrFtrPos));
@@ -424,6 +535,7 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 // Now find the first Non-strux frag within this hdrftr
 //
 	bool bStop = false;
+	PT_DocPosition posLastStrux = 0;
 	while((pfFrag->getType() == pf_Frag::PFT_Strux) && (pfFrag != getFragments().getLast()) && !bStop)
 	{
 		const pf_Frag_Strux * pfs = static_cast<const pf_Frag_Strux *>(pfFrag);
@@ -433,11 +545,17 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 		}
 		else
 		{
+			UT_DEBUGMSG(("Adding strux %x of type %d at Pos %d to strux vector for delete \n",pfs,pfs->getStruxType(),pfs->getPos()));
+			posLastStrux = pfs->getPos();
 			vecFragStrux.addItem((void *) pfFrag);
 			pfFrag = pfFrag->getNext();
 		}
 	}
 	PT_DocPosition TextStartPos = getFragPosition(pfFrag);
+	if(TextStartPos == posLastStrux)
+	{
+		TextStartPos++;
+	}
 	UT_DEBUGMSG(("SEVIOR: Deleting hdrftr Text Start Pos = %d \n",TextStartPos));
 //
 // Now find the end of the text in the header/footer
@@ -458,33 +576,53 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 				pfFrag = pfFrag->getNext();
 			}
 		}
-		PT_DocPosition TextEndPos = getFragPosition(pfFrag);
+		PT_DocPosition TextEndPos = 0;
+		TextEndPos = getFragPosition(pfFrag);
+		if(pfFrag == getFragments().getLast())
+		{
+			TextEndPos = getFragPosition(pfFrag->getPrev()) + pfFrag->getPrev()->getLength();
+		}
 		UT_DEBUGMSG(("SEVIOR: Deleting hdrftr Text End Pos = %d \n",TextEndPos));
 //
 // OK delete the text
 //
 		if(TextEndPos > TextStartPos)
 		{
-			deleteSpan(TextStartPos,TextEndPos,NULL,true);
+			UT_uint32 iRealDeleteCount;
+			deleteSpan(TextStartPos,TextEndPos,NULL,iRealDeleteCount,true);
+			// TODO -- is this right with revisions ???
 		}
 	}
 //
 // Now delete the struxes at the start.
 //
-// I assume there are blocks then sections. First delete all the blocks, then the
-// section strux.
 //
 	UT_uint32 count = vecFragStrux.getItemCount();
 	UT_ASSERT(count > 1);
 	UT_uint32 i=0;
 	bool bres = false;
+//
+// First delete the HdrFtr strux, then delete the blocks, this will enable the
+// the HdrFtr to be properly recreated on undo (Since it needs blocks to be
+// present before it can be created.)
+//
+	bres = _deleteStruxWithNotify(pfFragStruxHdrFtr->getPos(),pfFragStruxHdrFtr,NULL,NULL);
+	m_fragments.cleanFrags();
 	for(i=1; i<count; i++)
 	{
 		pf_Frag_Strux * pfs = (pf_Frag_Strux *) vecFragStrux.getNthItem(i);
-		bres = _deleteStruxWithNotify(pfs->getPos(),pfs,NULL,NULL);
+		if(static_cast<pf_Frag *>(pfs) ==  getFragments().getLast())
+		{
+			UT_DEBUGMSG(("Delete Last Strux type %d \n",pfs->getStruxType()));
+			UT_ASSERT(0);
+		}
+		UT_DEBUGMSG(("Delete Strux at %d strux type is %d \n",pfs->getPos(),pfs->getStruxType()));
+		if(pfs->getStruxType() != PTX_SectionHdrFtr)
+		{
+			bres = _deleteStruxWithNotify(pfs->getPos(),pfs,NULL,NULL);
+		}
 		UT_ASSERT(bres);
 	}
-	bres = _deleteStruxWithNotify(pfFragStruxHdrFtr->getPos(),pfFragStruxHdrFtr,NULL,NULL);
 	UT_ASSERT(bres);
 //	deleteSpan(HdrFtrPos,TextStartPos,NULL,true);
 }
