@@ -1,19 +1,21 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -45,8 +47,8 @@
 /*****************************************************************/
 /*****************************************************************/
 
-pt_PieceTable::pt_PieceTable(PD_Document * pDocument) 
-  : m_hashStyles(11), m_pts(PTS_Create), m_pDocument(pDocument),
+pt_PieceTable::pt_PieceTable(PD_Document * pDocument)
+  : m_pts(PTS_Create), m_hashStyles(11), m_pDocument(pDocument),
     m_atomicGlobCount(0)
 {
 
@@ -83,6 +85,66 @@ void pt_PieceTable::setPieceTableState(PTState pts)
 	m_varset.setPieceTableState(pts);
 }
 
+/*!
+ * Use this for deleting unneeded strux during doc import. Particularly useful for importing
+ * RTF.
+ */
+bool pt_PieceTable::deleteStruxNoUpdate(PL_StruxDocHandle sdh)
+{
+	const pf_Frag_Strux * pfs = static_cast<const pf_Frag_Strux *>(sdh);
+	UT_DEBUGMSG(("SEVIOR: deleting strux no update %x \n",sdh));
+	pf_Frag * pf = pfs->getNext();
+	if(pf != NULL && pf->getType() == pf_Frag::PFT_FmtMark)
+	{
+		getFragments().unlinkFrag(pf);
+		delete pf;
+	}
+	getFragments().unlinkFrag(const_cast<pf_Frag *>(static_cast<const pf_Frag *>(pfs)));
+	delete pfs;
+	return true;
+}
+
+/*!
+ * This method inserts a strux of type pts immediately before the sdh given.
+ * Attributes of the strux can be optionally passed. This method does not throw
+ * a change record and should only be used under exceptional circumstances to 
+ * repair the piecetable during loading. It was necessary to import RTF tables.
+ */
+bool pt_PieceTable::insertStruxNoUpdateBefore(PL_StruxDocHandle sdh, PTStruxType pts,const XML_Char ** attributes )
+{
+	const pf_Frag_Strux * pfs = static_cast<const pf_Frag_Strux *>(sdh);
+	UT_DEBUGMSG(("SEVIOR: Inserting strux of type %d no update %x \n",pts,sdh));
+//
+// Create an indexAP
+//
+	PT_AttrPropIndex indexAP = pfs->getIndexAP();
+	if(attributes)
+	{		
+		PT_AttrPropIndex pAPIold = indexAP;
+		bool bMerged = m_varset.mergeAP(PTC_AddFmt,pAPIold,attributes,NULL,&indexAP,getDocument());
+		UT_ASSERT(bMerged);
+	}
+//
+// create a strux
+//
+	pf_Frag_Strux * pNewStrux = NULL;
+	_createStrux(pts,indexAP,&pNewStrux);
+//
+// Insert it.
+//
+	pf_Frag * pfPrev = pfs->getPrev();
+	UT_ASSERT(pfPrev);
+	if(!pfPrev)
+	{
+		return false;
+	}
+	m_fragments.insertFrag(pfPrev,static_cast<pf_Frag *>(pNewStrux));
+#if 0
+	m_pDocument->miniDump(sdh,8);
+#endif
+	return true;
+}
+
 void pt_PieceTable::_unlinkFrag(pf_Frag * pf,
 								pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
@@ -96,7 +158,7 @@ void pt_PieceTable::_unlinkFrag(pf_Frag * pf,
 		*ppfEnd = pf->getNext();
 	if (pfragOffsetEnd)
 		*pfragOffsetEnd = 0;
-	
+
 	pf_Frag * pp = pf->getPrev();
 
 	m_fragments.unlinkFrag(pf);
@@ -159,7 +221,7 @@ bool pt_PieceTable::_getSpanAttrPropHelper(pf_Frag * pf, const PP_AttrProp ** pp
 
 	case pf_Frag::PFT_Text:
 		ReturnThis( pf_Frag_Text, pf );
-		
+
 	case pf_Frag::PFT_Object:
 		ReturnThis( pf_Frag_Object, pf );
 
@@ -172,20 +234,20 @@ bool pt_PieceTable::_getSpanAttrPropHelper(pf_Frag * pf, const PP_AttrProp ** pp
 #undef ReturnThis
 }
 
-		  
+
 bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, bool bLeftSide,
 									   const PP_AttrProp ** ppAP) const
 {
 	// return the AP for the text at the given offset from the given strux.
 	// offset zero now refers to the first character in the block, so adding
 	// fl_BLOCK_STRUX_OFFSET to the offset in the call is no longer necessary.
-	
+
 	UT_ASSERT(sdh);
 	UT_ASSERT(ppAP);
 
-	pf_Frag * pf = (pf_Frag *)sdh;
+	const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
 	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
-	pf_Frag_Strux * pfsBlock = static_cast<pf_Frag_Strux *> (pf);
+	const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
 	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
 
 	UT_uint32 cumOffset = 0;
@@ -197,7 +259,7 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 
 		if (offset > cumEndOffset)		// the place we want is way past the end of pfTemp,
 			continue;					// so keep searching.
-		
+
 		if (offset == cumOffset)		// there's a frag boundary exactly where we want. pfTemp is to our right.
 		{
 			// FmtMarks have length zero, so we have to see what side of the position the caller wants.
@@ -208,12 +270,12 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 		}
 
 		UT_ASSERT(offset > cumOffset);
-		
+
 		if (offset == cumEndOffset)		// there's a frag boundary exactly where we want. pfTemp is to our left.
 		{
 			if (!bLeftSide)
 				continue;				// return the next one on the next loop iteration
-			
+
 			// FmtMarks have length zero, so we advance to put it to our left and then decide what to do
 			if (pfTemp->getNext() && (pfTemp->getNext()->getType()==pf_Frag::PFT_FmtMark))
 				continue;				// we'll return this one on the next loop iteration
@@ -232,6 +294,7 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 	return false;
 }
 
+
 bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 								  const UT_UCSChar ** ppSpan, UT_uint32 * pLength) const
 {
@@ -240,23 +303,33 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 
 	*ppSpan = NULL;
 	*pLength = 0;
-	
-	pf_Frag * pf = (pf_Frag *)sdh;
-	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
-	pf_Frag_Strux * pfsBlock = static_cast<pf_Frag_Strux *> (pf);
-	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
 
+	const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
+	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
+	const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
+	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
+	xxx_UT_DEBUGMSG(("getSpanPtr: Requested offset %d \n",offset));
+	
 	UT_uint32 cumOffset = pf->getLength();
 	for (pf_Frag * pfTemp=pfsBlock->getNext(); (pfTemp); pfTemp=pfTemp->getNext())
 	{
+		xxx_UT_DEBUGMSG(("getSpanPtr: offset %d cumOffset %d \n",offset,cumOffset));
 		if (offset == cumOffset)
 		{
 			if (pfTemp->getType() == pf_Frag::PFT_FmtMark)
 				continue;
-			
+			if(isFootnote(pfTemp) || isEndFootnote(pfTemp))
+			{
+				cumOffset += pfTemp->getLength();
+				continue;
+			}
 			if (pfTemp->getType() != pf_Frag::PFT_Text)
+			{
+				xxx_UT_DEBUGMSG(("getSpanPtr: Error 1 offset %d cumOffset %d \n",offset,cumOffset));
+//				UT_ASSERT(0);
 				return false;
-			
+			}
+
 			pf_Frag_Text * pfText = static_cast<pf_Frag_Text *> (pfTemp);
 			*ppSpan = getPointer(pfText->getBufIndex());
 			*pLength = pfText->getLength();
@@ -264,9 +337,16 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 		}
 		if (offset < cumOffset+pfTemp->getLength())
 		{
+			if(isFootnote(pfTemp) || isEndFootnote(pfTemp))
+			{
+				cumOffset += pfTemp->getLength();
+				continue;
+			}
 			if (pfTemp->getType() != pf_Frag::PFT_Text)
+			{
+				xxx_UT_DEBUGMSG(("getSpanPtr: Error 2 offset %d cumOffset %d \n",offset,cumOffset));
 				return false;
-
+			}
 			pf_Frag_Text * pfText = static_cast<pf_Frag_Text *> (pfTemp);
 			const UT_UCSChar * p = getPointer(pfText->getBufIndex());
 			UT_uint32 delta = offset - cumOffset;
@@ -277,9 +357,11 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 
 		cumOffset += pfTemp->getLength();
 	}
+	xxx_UT_DEBUGMSG(("getSpanPtr: Error 3 offset %d cumOffset %d \n",offset,cumOffset));
 	return false;
 }
 
+
 PD_Document * pt_PieceTable::getDocument(void)
 {
         return m_pDocument;
@@ -294,19 +376,20 @@ PD_Document * pt_PieceTable::getDocument(void)
   Copy the contents (unicode character data) of the paragraph (block)
   into the growbuf given.  We append the content onto the growbuf.
 */
-bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh, 
+bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
                                    UT_GrowBuf * pgb) const
 {
     UT_ASSERT(pgb);
-	
-    pf_Frag * pf = (pf_Frag *)sdh;
-    UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
-    pf_Frag_Strux * pfsBlock = static_cast<pf_Frag_Strux *> (pf);
-    UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
+
+    const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
+    UT_return_val_if_fail(pf->getType() == pf_Frag::PFT_Strux, false);
+    const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
+    UT_return_val_if_fail(pfsBlock->getStruxType() == PTX_Block, false);
 
     UT_uint32 bufferOffset = pgb->getLength();
-	
+
     pf_Frag * pfTemp = pfsBlock->getNext();
+	UT_sint32 countFoots = 0;
     while (pfTemp)
     {
         switch (pfTemp->getType())
@@ -314,24 +397,67 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
         default:
             UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
         case pf_Frag::PFT_Strux:
+//
+// Have to deal with embedded section struxes like Footnotes. We do this by
+// filling the block buffer with the content contained until we find an
+// end of embedded container. By placing zero's and content as expected in
+// the block buffer we match the situation in fl_BlockLayout.
+//
+		{
+			UT_GrowBufElement valz = 0;
+            bool bAppended;
+			if(isFootnote(pfTemp))
+			{
+				countFoots++;
+				bAppended = pgb->ins(bufferOffset,&valz,1);
+				UT_ASSERT(bAppended);
+				bufferOffset++;
+				pfTemp = pfTemp->getNext();
+				break;
+			}
+			if(isEndFootnote(pfTemp))
+			{
+				countFoots--;
+				if(countFoots < 0)
+				{
+					pfTemp = NULL;
+					break;
+				}
+				bAppended = pgb->ins(bufferOffset,&valz,1);
+				UT_ASSERT(bAppended);
+				bufferOffset++;
+				pfTemp = pfTemp->getNext();
+				break;
+			}
+			if(countFoots>0)
+			{
+				bAppended = pgb->ins(bufferOffset,&valz,1);
+				UT_ASSERT(bAppended);
+				bufferOffset++;
+				pfTemp = pfTemp->getNext();
+				break;
+			}
+			pfTemp = NULL;
+			break;
+		}
         case pf_Frag::PFT_EndOfDoc:
             pfTemp = NULL;
             break;
-            
+
         case pf_Frag::PFT_FmtMark:
             pfTemp = pfTemp->getNext();
             break;
-            
+
         case pf_Frag::PFT_Text:
         {
             pf_Frag_Text * pft = static_cast<pf_Frag_Text *>(pfTemp);
             const UT_UCSChar * pSpan = getPointer(pft->getBufIndex());
             UT_uint32 length = pft->getLength();
-            
+
             bool bAppended;
-            bAppended = pgb->ins(bufferOffset,pSpan,length);
+            bAppended = pgb->ins(bufferOffset,reinterpret_cast<const UT_GrowBufElement*>(pSpan),length);
             UT_ASSERT(bAppended);
-            
+
             bufferOffset += length;
         }
         pfTemp = pfTemp->getNext();
@@ -349,7 +475,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
               buffer.  Obviously, those codes aren't useful, but at
               least the app doesn't crash, and the rest of the text in
               the block is safely stored in the buffer in the proper
-              location. 
+              location.
 
               The UCS_ABI_OBJECT used to be defined as a space, but
               that caused selection code to fail for fields since the
@@ -357,25 +483,25 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
               spaces. Now the UCS_ABI_OBJECT is instead defined as an
               alpha character. Doesn't really matter since it'll never
               be used for anything but limit checking anyway. See bug
-              #223 for details. 
+              #223 for details.
 			*/
 
             UT_uint32 length = pfTemp->getLength();
-            
+
             // TODO investigate appending the SPACES directly to
             // TODO the pgb.  **or** investigate the cost of this
             // TODO malloc and what happens when it fails....
-				
+
             UT_UCSChar* pSpaces = new UT_UCSChar[length];
             for (UT_uint32 i=0; i<length; i++)
             {
                 pSpaces[i] = UCS_ABI_OBJECT;
             }
             bool bAppended;
-            bAppended = pgb->ins(bufferOffset, pSpaces, length);
+            bAppended = pgb->ins(bufferOffset, reinterpret_cast<UT_GrowBufElement*>(pSpaces), length);
             delete[] pSpaces;
             UT_ASSERT(bAppended);
-		
+
             bufferOffset += length;
         }
         pfTemp = pfTemp->getNext();
@@ -414,12 +540,17 @@ PT_DocPosition pt_PieceTable::getStruxPosition(PL_StruxDocHandle sdh) const
 {
 	// return absolute document position of the given handle.
 
-	pf_Frag * pfToFind = (pf_Frag *)sdh;
+	const pf_Frag * pfToFind = static_cast<const pf_Frag *>(sdh);
 
 	return getFragPosition(pfToFind);
 }
 
 void pt_PieceTable::deleteHdrFtrStrux(pf_Frag_Strux * pfs)
+{
+	_realDeleteHdrFtrStrux(pfs);
+}
+
+void pt_PieceTable::_realDeleteHdrFtrStrux(pf_Frag_Strux * pfs)
 {
 	_deleteHdrFtrStruxWithNotify(pfs);
 }
@@ -440,25 +571,25 @@ bool pt_PieceTable::getFragFromPosition(PT_DocPosition docPos,
 // Sevior do a binary search here now
 //
 	pf_Frag * pfLast = m_fragments.findFirstFragBeforePos(docPos);
-
-	// why not just:
-	// UT_ASSERT(pfLast); *ppf = pfLast; if (pFragOffset) *pFragOffset = docPos - pfLast->getPos();
 	if(pfLast)
 	{
-		while(pfLast->getNext() && docPos >= pfLast->getPos() + pfLast->getLength())
+		xxx_UT_DEBUGMSG(("_findFrag: docPos %d pfLast->getPos %d pfLast->getLength %d \n",docPos,pfLast->getPos(),pfLast->getLength()));
+		while(pfLast->getNext() && docPos >= (pfLast->getPos() + pfLast->getLength()))
 		{
+			xxx_UT_DEBUGMSG(("_findFrag: docPos %d pfLast->getPos %d pfLast->getLength %d \n",docPos,pfLast->getPos(),pfLast->getLength()));
 			pfLast = pfLast->getNext();
 		}
+		xxx_UT_DEBUGMSG(("_findFrag: docPos %d pfLast->getPos %d pfLast->getLength %d offset %d Frag Type %d \n",docPos,pfLast->getPos(),pfLast->getLength(),docPos - pfLast->getPos(),pfLast->getType()));
 		if (pFragOffset)
 			*pFragOffset = docPos - pfLast->getPos();
-		*ppf = pfLast;
+		*ppf = pfLast;       
 		return true;
 		xxx_UT_DEBUGMSG(("SEVIOR: I found frag %d \n",getFragNumber(pfLast)));
 	}
 
 	UT_ASSERT(pfLast);
 	UT_ASSERT(pfLast->getType() == pf_Frag::PFT_EndOfDoc);
-	
+
 	return true;
 }
 	//  PT_DocPosition sum = 0;
@@ -474,10 +605,10 @@ bool pt_PieceTable::getFragFromPosition(PT_DocPosition docPos,
 
 //  			// a FmtMark has length zero.  we don't want to find it
 //  			// in this loop -- rather we want the thing just past it.
-			
+
 //  			UT_ASSERT(pf->getType() != pf_Frag::PFT_FmtMark);
 //  			UT_DEBUGMSG(("SEVIOR: Jeff finds frag %d at Pos %d \n",getFragNumber(pf),docPos));
-			
+
 //  			return true;
 //  		}
 
@@ -495,7 +626,7 @@ bool pt_PieceTable::getFragFromPosition(PT_DocPosition docPos,
 //  	UT_ASSERT(pfLast->getType() == pf_Frag::PFT_EndOfDoc);
 
 //  	// TODO if (docPos > sum) we should probably complain...
-	
+
 //  	*ppf = pfLast;
 //  	if (pFragOffset)
 //  		*pFragOffset = docPos - sum;
@@ -510,13 +641,13 @@ bool pt_PieceTable::getFragsFromPositions(PT_DocPosition dPos1, PT_DocPosition d
 											 pf_Frag ** ppf2, PT_BlockOffset * pOffset2) const
 {
 	// compute the (fragment,offset) pairs for each position given.
-	
+
 	UT_ASSERT(dPos1 <= dPos2);
 	UT_ASSERT(ppf1);
 	UT_ASSERT(pOffset1);
-	
+
 	// the first set has to be done the hard way.
-	
+
 	if (!getFragFromPosition(dPos1,ppf1,pOffset1))
 		return false;
 
@@ -546,7 +677,7 @@ bool pt_PieceTable::getFragsFromPositions(PT_DocPosition dPos1, PT_DocPosition d
 		*pOffset2 = offset+deltaPos;
 	return true;
 }
-	
+
 bool pt_PieceTable::getStruxFromPosition(PL_ListenerId listenerId,
 											PT_DocPosition docPos,
 											PL_StruxFmtHandle * psfh) const
@@ -557,7 +688,7 @@ bool pt_PieceTable::getStruxFromPosition(PL_ListenerId listenerId,
 	pf_Frag_Strux * pfs = NULL;
 	if (!_getStruxFromPosition(docPos,&pfs))
 		return false;
-	
+
 	*psfh = pfs->getFmtHandle(listenerId);
 	return true;
 }
@@ -573,7 +704,7 @@ bool pt_PieceTable::getStruxOfTypeFromPosition(PL_ListenerId listenerId,
 	pf_Frag_Strux * pfs = NULL;
 	if (!_getStruxOfTypeFromPosition(docPos,pts,&pfs))
 		return false;
-	
+
 	*psfh = pfs->getFmtHandle(listenerId);
 	return true;
 }
@@ -582,60 +713,86 @@ bool pt_PieceTable::getStruxOfTypeFromPosition(PL_ListenerId listenerId,
 /// immediately prior to the given absolute document position.
 ///
 bool pt_PieceTable::getStruxOfTypeFromPosition( PT_DocPosition docPos,
-						   PTStruxType pts, 
+						   PTStruxType pts,
 						   PL_StruxDocHandle * sdh) const
 {
 
 	pf_Frag_Strux * pfs = NULL;
 	if (!_getStruxOfTypeFromPosition(docPos,pts,&pfs))
-		return false;
-	*sdh = ( PL_StruxDocHandle ) pfs;
+			return false;
+	*sdh = static_cast<PL_StruxDocHandle >(pfs);
 	return true;
 }
 
+bool pt_PieceTable::isEndFootnote(pf_Frag * pf) const
+{
+	if(pf->getType() == pf_Frag::PFT_Strux)
+	{
+		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
+		if((pfs->getStruxType() == PTX_EndFootnote) || (pfs->getStruxType() == PTX_EndEndnote))
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
+
+bool pt_PieceTable::isFootnote(pf_Frag * pf) const
+{
+	if(pf->getType() == pf_Frag::PFT_Strux)
+	{
+		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
+		if((pfs->getStruxType() == PTX_SectionFootnote) || (pfs->getStruxType() == PTX_SectionEndnote))
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
 bool pt_PieceTable::_getStruxFromPosition(PT_DocPosition docPos,
-											 pf_Frag_Strux ** ppfs) const
+											 pf_Frag_Strux ** ppfs,
+                                              bool bSkipFootnotes) const
 {
 	// return the strux fragment immediately prior (containing)
-	// the given absolute document position.
+	// the given absolute document position.  If bSkip set, skip past
+    // Footnote struxes.
+	UT_sint32 countEndFootnotes = 0;
 	pf_Frag * pfFirst = m_fragments.findFirstFragBeforePos( docPos);
-	while(pfFirst && pfFirst->getPrev() && pfFirst->getPos()  >= docPos)
+
+	if(isEndFootnote(pfFirst))
+		countEndFootnotes++;
+
+	while(pfFirst && pfFirst->getPrev() && pfFirst->getPos() >= docPos)
 	{
 		pfFirst = pfFirst->getPrev();
+		if (isFootnote(pfFirst))
+			countEndFootnotes--;
+		else if(isEndFootnote(pfFirst))
+			countEndFootnotes++;
+
+		xxx_UT_DEBUGMSG(("countEndNotes 1 %d \n",countEndFootnotes));
 	}
-	while(pfFirst && pfFirst->getPrev() && pfFirst->getType() !=pf_Frag::PFT_Strux)
+	while(pfFirst && pfFirst->getPrev() && 
+		  (pfFirst->getType() != pf_Frag::PFT_Strux || 
+		   (bSkipFootnotes && ((countEndFootnotes > 0) || isFootnote(pfFirst) || isEndFootnote(pfFirst)))))
 	{
 		pfFirst = pfFirst->getPrev();
+		if(pfFirst == NULL)
+		{
+			break;
+		}
+		if(isFootnote(pfFirst))
+			countEndFootnotes--;
+		else if(isEndFootnote(pfFirst))
+			countEndFootnotes++;
+		xxx_UT_DEBUGMSG(("countEndNotes 2 %d \n",countEndFootnotes));
 	}
-//	UT_DEBUGMSG(("SEVIOR: I find strux frag number %d  sum = %d at pos %d \n",getFragNumber(pfFirst),pfFirst->getPos(),docPos));
+	xxx_UT_DEBUGMSG(("countEndNotes final %d \n",countEndFootnotes));
   	pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *> (pfFirst);
   	*ppfs = pfs;
     return true;
-
-	//  PT_DocPosition sum = pfFirst->getPos();
-//  	pf_Frag * pfLastStrux = NULL;
-
-//  	for (pf_Frag * pf = pfFirst; (pf); pf=pf->getNext())
-//  	{
-//  		if (pf->getType() == pf_Frag::PFT_Strux)
-//  			pfLastStrux = pf;
-
-//  		sum += pf->getLength();
-
-//  		if (sum >= docPos)
-//  			goto FoundIt;
-//  	}
-
-	// if we fall out of the loop, we didn't have a text node
-	// at or around the document position requested.
-	// return the last strux in the document.
-
-//   FoundIt:
-
-//  	pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *> (pfLastStrux);
-//  	UT_DEBUGMSG(("SEVIOR: Jeff find strux frag number %d  sum = %d at pos %d \n",getFragNumber(pfLastStrux),sum,docPos));
-//  	*ppfs = pfs;
-//      return true;
 }
 
 bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
@@ -645,33 +802,54 @@ bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
 	// return the strux fragment of the given type containing
 	// the given absolute document position.
 	UT_ASSERT(ppfs);
+	xxx_UT_DEBUGMSG(("_getStruxOfTypeFromPosition: looking for type %d \n",pts));
 	*ppfs = NULL;
-	
+
 	pf_Frag_Strux * pfs = NULL;
-	if (!_getStruxFromPosition(dpos,&pfs))
+	if (!_getStruxFromPosition(dpos,&pfs, !(pts == PTX_EndFootnote || pts == PTX_SectionFootnote)))
 		return false;
 
-	if (pfs->getStruxType() == pts || (pts == PTX_Section && pfs->getStruxType() == PTX_SectionHdrFtr))		// is it of the type we want
+	PTStruxType pfsType = pfs->getStruxType();
+	if (pfsType == pts || (pts == PTX_Section && pfsType == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsType == PTX_SectionFootnote) || (pts == PTX_SectionEndnote && pfsType == PTX_SectionEndnote) || (pts == PTX_SectionTable && pfsType == PTX_SectionTable) || (pts == PTX_SectionCell && pfsType == PTX_SectionCell) || (pts == PTX_EndTable && pfsType == PTX_EndTable) || (pts == PTX_EndCell && pfsType == PTX_EndCell)  )		// is it of the type we want
 	{
 		*ppfs = pfs;
 		return true;
 	}
 
 	// if not, we walk backwards thru the list and try to find it.
-
+	UT_sint32 numEndTable = 0;
 	for (pf_Frag * pf=pfs; (pf); pf=pf->getPrev())
 		if (pf->getType() == pf_Frag::PFT_Strux)
 		{
 			pf_Frag_Strux * pfsTemp = static_cast<pf_Frag_Strux *>(pf);
-			if (pfsTemp->getStruxType() == pts || (pts == PTX_Section && pfsTemp->getStruxType() == PTX_SectionHdrFtr))	// did we find it
+			if(pfsTemp->getStruxType() == PTX_EndTable)
 			{
-				*ppfs = pfsTemp;
-				return true;
+				numEndTable++;
+			}
+			else if(pfsTemp->getStruxType() == PTX_SectionTable)
+			{
+				numEndTable--;
+			}
+			if (pfsTemp->getStruxType() == pts || (pts == PTX_Section && pfsTemp->getStruxType() == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsTemp->getStruxType() == PTX_SectionFootnote) || (pts == PTX_EndFootnote && pfsTemp->getStruxType() == PTX_EndFootnote) || (pts == PTX_SectionEndnote && pfsTemp->getStruxType() == PTX_SectionEndnote) || (pts == PTX_EndEndnote && pfsTemp->getStruxType() == PTX_EndEndnote))	// did we find it
+			{
+				if(((numEndTable < 0) && (pfsTemp->getStruxType()==PTX_SectionTable)) || (numEndTable == 0 && (pfsTemp->getStruxType()!=PTX_SectionTable)))
+				{
+					*ppfs = pfsTemp;
+					return true;
+				}
+				else if(pfsTemp->getStruxType() != PTX_SectionTable &&
+						pfsTemp->getStruxType() != PTX_SectionCell &&
+						pfsTemp->getStruxType() != PTX_EndTable &&
+						pfsTemp->getStruxType() != PTX_EndCell)
+				{
+					*ppfs = pfsTemp;
+					return true;
+				}
 			}
 		}
 
 	// did not find it.
-	
+
 	return false;
 }
 
@@ -693,6 +871,48 @@ bool pt_PieceTable::_getStruxFromFrag(pf_Frag * pfStart, pf_Frag_Strux ** ppfs)
 	return true;
 }
 
+
+bool pt_PieceTable::_getStruxFromFragSkip(pf_Frag * pfStart, pf_Frag_Strux ** ppfs) const
+{
+	// return the strux frag immediately prior to (containing)
+	// the given fragment while skipping endFootnote/footnote stuff.
+
+	*ppfs = NULL;
+
+	pf_Frag * pf;
+	UT_sint32 countFoots = 0;
+	if(isEndFootnote(pfStart))
+	{
+		countFoots++;
+	}
+	pf = pfStart->getPrev();
+	if(isEndFootnote(pf))
+	{
+		countFoots++;
+	}
+	xxx_UT_DEBUGMSG(("_getStruxFromFragStrux: 1 countFoots %d \n",countFoots));
+	while(pf && ((pf->getType() != pf_Frag::PFT_Strux) || (countFoots > 0) 
+				 || isFootnote(pf) || isEndFootnote(pf)))
+	{
+		pf=pf->getPrev();
+		if(isFootnote(pf))
+		{
+			countFoots--;
+		}
+		else if(isEndFootnote(pf))
+		{
+			countFoots++;
+		}
+		xxx_UT_DEBUGMSG(("_getStruxFromFragStrux: 2 countFoots %d \n",countFoots));
+	}
+		;
+	if (!pf)
+		return false;
+
+	*ppfs = static_cast<pf_Frag_Strux *>(pf);
+	return true;
+}
+
 UT_uint32 pt_PieceTable::_computeBlockOffset(pf_Frag_Strux * pfs,pf_Frag * pfTarget) const
 {
 	// return the block offset of the beginning of pfTarget from the end of pfs.
@@ -706,7 +926,7 @@ UT_uint32 pt_PieceTable::_computeBlockOffset(pf_Frag_Strux * pfs,pf_Frag * pfTar
 	return sum;
 }
 
-	
+
 void pt_PieceTable::clearIfAtFmtMark(PT_DocPosition dpos)
 {
 	while (_lastUndoIsThisFmtMark(dpos))
