@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -25,20 +25,23 @@
 #include "ut_types.h"
 #include "ut_string.h"
 
+#include "xap_App.h"
 #include "xap_Preview.h"
 
 #include "ap_Preview_Paragraph.h"
 #include "ap_Strings.h"
 #include "ap_Dialog_Lists.h"
 
+#include <fribidi.h>
+
 /************************************************************************/
 
 // all of these are measured in pixels
 
-#define DEFAULT_TOP_MARGIN 5
+#define DEFAULT_TOP_MARGIN m_gc->tlu(5)
 
-#define DEFAULT_LEFT_STOP 20
-#define DEFAULT_RIGHT_STOP 20
+#define DEFAULT_LEFT_STOP m_gc->tlu(20)
+#define DEFAULT_RIGHT_STOP m_gc->tlu(20)
 
 #define DEFAULT_BEFORE_SPACING 0
 #define DEFAULT_AFTER_SPACING 0
@@ -50,7 +53,7 @@ AP_Preview_Paragraph_Block::AP_Preview_Paragraph_Block(UT_RGBColor & clr,
 													   UT_uint32 fontHeight)
 {
 	UT_ASSERT(gc);
-	
+
 	m_clr.m_red = clr.m_red;
 	m_clr.m_grn = clr.m_grn;
 	m_clr.m_blu = clr.m_blu;
@@ -68,7 +71,7 @@ AP_Preview_Paragraph_Block::AP_Preview_Paragraph_Block(UT_RGBColor & clr,
 	m_align = AP_Dialog_Paragraph::align_LEFT;
 	m_indent = AP_Dialog_Paragraph::indent_NONE;
 	m_spacing = AP_Dialog_Paragraph::spacing_SINGLE;
-	
+
 	m_fontHeight = fontHeight;
 }
 
@@ -100,11 +103,11 @@ void AP_Preview_Paragraph_Block::setText(const UT_UCSChar * text)
 
 	// clear the widths vector (has no memory at each item)
 	m_widths.clear();
-	
+
 	// dup the string for harmful chunkification
 	UT_UCSChar * clone = NULL;
-	UT_UCS_cloneString(&clone, text);
-	
+	UT_UCS4_cloneString(&clone, text);
+
 	UT_UCSChar * i = clone;
 
 	while (*i != 0)
@@ -119,7 +122,7 @@ void AP_Preview_Paragraph_Block::setText(const UT_UCSChar * text)
 			m_words.addItem(clone);
 
 			// measure clone item
-			m_widths.addItem((void *) m_gc->measureString(clone, 0, UT_UCS_strlen(clone), NULL));
+			m_widths.addItem((void *) m_gc->measureString(clone, 0, UT_UCS4_strlen(clone), NULL));
 
 			// advance clone pointer for new word
 			clone = i + 1;
@@ -130,7 +133,7 @@ void AP_Preview_Paragraph_Block::setText(const UT_UCSChar * text)
 	// add last word
 	m_words.addItem(clone);
 	// measure last word
-	m_widths.addItem((void *) m_gc->measureString(clone, 0, UT_UCS_strlen(clone), NULL));
+	m_widths.addItem((void *) m_gc->measureString(clone, 0, UT_UCS4_strlen(clone), NULL));
 }
 
 // ignores NULL parameters, otherwise scales dimensioned strings into
@@ -162,7 +165,7 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 
 	if(pageLeftMargin)
 	{
-		m_leftStop = SCALE_TO_PIXELS(pageLeftMargin);
+		m_leftStop = m_gc->tlu(SCALE_TO_PIXELS(pageLeftMargin));
 	}
 	else
 	{
@@ -172,14 +175,14 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 	// left margins are in or out from the default stop
 	if (leftIndent)
 	{
-		m_leftStop += SCALE_TO_PIXELS(leftIndent);
+		m_leftStop += m_gc->tlu(SCALE_TO_PIXELS(leftIndent));
 		// NOTE : if we recomputed the leftIndent, we have to recompute
 		// NOTE : the firstLineLeftStop below
 	}
-	
+
 	if(pageRightMargin)
 	{
-		m_rightStop = SCALE_TO_PIXELS(pageRightMargin);
+		m_rightStop = m_gc->tlu(SCALE_TO_PIXELS(pageRightMargin));
 	}
 	else
 	{
@@ -188,10 +191,12 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 
 	// right margins are in or out from the default stop
 	if (rightIndent)
-		m_rightStop += SCALE_TO_PIXELS(rightIndent);
+		m_rightStop += m_gc->tlu(SCALE_TO_PIXELS(rightIndent));
 
 	STORE_CONVERTED(m_beforeSpacing, beforeSpacing);
 	STORE_CONVERTED(m_afterSpacing, afterSpacing);
+	m_beforeSpacing = m_gc->tlu(m_beforeSpacing);
+	m_afterSpacing = m_gc->tlu(m_afterSpacing);
 
 	m_indent = indent;
 	switch (m_indent)
@@ -199,10 +204,10 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 		// the signage for these two is handled in the conversion through
 		// UT_convertToInches()
 	case AP_Dialog_Paragraph::indent_FIRSTLINE:
-		m_firstLineLeftStop = m_leftStop + SCALE_TO_PIXELS(firstLineIndent);
+		m_firstLineLeftStop = m_leftStop + m_gc->tlu(SCALE_TO_PIXELS(firstLineIndent));
 		break;
 	case AP_Dialog_Paragraph::indent_HANGING:
-		m_firstLineLeftStop = m_leftStop - SCALE_TO_PIXELS(firstLineIndent);
+		m_firstLineLeftStop = m_leftStop - m_gc->tlu(SCALE_TO_PIXELS(firstLineIndent));
 		break;
 	case AP_Dialog_Paragraph::indent_NONE:
 		m_firstLineLeftStop = m_leftStop;
@@ -228,19 +233,19 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 		case AP_Dialog_Paragraph::spacing_ATLEAST:
 			// TODO : THIS IS BROKEN SOMEHOW.  m_lineSpacing should be the number
 			// TODO : of pixels needed to place before the line (of height
-			// TODO : m_fontHeight pixels).  
-			
+			// TODO : m_fontHeight pixels).
+
 			// we measure from top to top here, and use a minimum of the current
 			// line height
-			if (SCALE_TO_PIXELS(lineSpacing) > m_fontHeight)
-				m_lineSpacing = SCALE_TO_PIXELS(lineSpacing) - m_fontHeight;
+			if (m_gc->tlu(SCALE_TO_PIXELS(lineSpacing)) > static_cast<UT_sint32>(m_fontHeight))
+				m_lineSpacing = m_gc->tlu(SCALE_TO_PIXELS(lineSpacing)) - m_fontHeight;
 			else
 				m_lineSpacing = 0;
 			break;
 		case AP_Dialog_Paragraph::spacing_EXACTLY:
 			// for exact, we always give them exactly what they asked for,
 			// overlapping and all
-			m_lineSpacing = SCALE_TO_PIXELS(lineSpacing);
+			m_lineSpacing = m_gc->tlu(SCALE_TO_PIXELS(lineSpacing));
 			break;
 		case AP_Dialog_Paragraph::spacing_MULTIPLE:
 			m_lineSpacing = (UT_uint32) ((double) m_fontHeight
@@ -250,29 +255,29 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 	}
 
 }
-		
+
 
 /************************************************************************/
 
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 										   const UT_UCSChar * text,
 										   AP_Dialog_Lists * dlg)
-	: XAP_Preview(gc)
+  : XAP_Preview(gc), m_dir(FRIBIDI_TYPE_LTR)
 {
 	UT_ASSERT(text && dlg);
 
 	m_font = NULL;
 	m_fontHeight = 0;
-	
+
 	m_y = DEFAULT_TOP_MARGIN;
-		
+
 	m_clrWhite = new UT_RGBColor(255,255,255);
 	m_clrBlack = new UT_RGBColor(0,0,0);
 	m_clrGray = new UT_RGBColor(192,192,192);
 
 	// initialize font to start measuring with for following setText calls
 	_loadDrawFont();
-	
+
 	{
 		// this block is a dummy block
 		m_previousBlock = new AP_Preview_Paragraph_Block(*m_clrGray,
@@ -287,7 +292,7 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 									NULL,NULL,NULL,NULL,NULL,
 									AP_Dialog_Paragraph::spacing_SINGLE);
 	}
-	
+
 	{
 		// this block is our ACTIVE block
 		m_activeBlock = new AP_Preview_Paragraph_Block(*m_clrBlack,
@@ -302,14 +307,14 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_SPECIAL_INDENT),
 									(AP_Dialog_Paragraph::tIndentState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_SPECIAL_INDENT),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_LEFT_INDENT),
-									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_RIGHT_INDENT),								 
+									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_RIGHT_INDENT),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_BEFORE_SPACING),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_AFTER_SPACING),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_SPECIAL_SPACING),
 									(AP_Dialog_Paragraph::tSpacingState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_SPECIAL_SPACING));
 #endif
 	}
-	
+
 	{
 		// another dummy block
 		m_followingBlock = new AP_Preview_Paragraph_Block(*m_clrGray,
@@ -326,24 +331,18 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 	}
 
 	const XAP_StringSet * pSS = dlg->getApp()->getStringSet();
-	
-	UT_UCSChar * tmp = NULL;
 
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewPrevParagraph));
-	m_previousBlock->setText(tmp);
-	FREEP(tmp);
+	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
 
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewFollowParagraph));
-	m_followingBlock->setText(tmp);
-	FREEP(tmp);
+	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 					   const UT_UCSChar * text,
-					   XAP_Dialog * dlg) : XAP_Preview(gc)
+										   XAP_Dialog * dlg) : XAP_Preview(gc), m_dir(FRIBIDI_TYPE_LTR)
 {
   // this method heavily relies upon the parent dlg to call setFormat()
   // rather than auto-generating defaults
@@ -351,9 +350,9 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 
 	m_font = NULL;
 	m_fontHeight = 0;
-	
+
 	m_y = DEFAULT_TOP_MARGIN;
-		
+
 	m_clrWhite = new UT_RGBColor(255,255,255);
 	m_clrBlack = new UT_RGBColor(0,0,0);
 	m_clrGray = new UT_RGBColor(192,192,192);
@@ -386,41 +385,34 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 	}
 
 	const XAP_StringSet * pSS = dlg->getApp()->getStringSet();
-	
-	UT_UCSChar * tmp = NULL;
 
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewPrevParagraph));
-	m_previousBlock->setText(tmp);
-	FREEP(tmp);
+	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
 
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewFollowParagraph));
-	m_followingBlock->setText(tmp);
-	FREEP(tmp);
-	  
+	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 					   const UT_UCSChar * text,
 					   AP_Dialog_Paragraph * dlg)
-	: XAP_Preview(gc)
+	: XAP_Preview(gc),m_dir(FRIBIDI_TYPE_LTR)
 {
 	UT_ASSERT(text && dlg);
 
 	m_font = NULL;
 	m_fontHeight = 0;
-	
+
 	m_y = DEFAULT_TOP_MARGIN;
-		
+
 	m_clrWhite = new UT_RGBColor(255,255,255);
 	m_clrBlack = new UT_RGBColor(0,0,0);
 	m_clrGray = new UT_RGBColor(192,192,192);
 
 	// initialize font to start measuring with for following setText calls
 	_loadDrawFont();
-	
+
 	{
 		// this block is a dummy block
 		m_previousBlock = new AP_Preview_Paragraph_Block(*m_clrGray,
@@ -429,13 +421,13 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 														 m_fontHeight);
 		m_previousBlock->setFormat(dlg->m_pageLeftMargin,
 									dlg->m_pageRightMargin,
-									AP_Dialog_Paragraph::align_LEFT,
+									(AP_Dialog_Paragraph::tAlignState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_ALIGNMENT),
 									NULL,
 									AP_Dialog_Paragraph::indent_NONE,
 									NULL,NULL,NULL,NULL,NULL,
 									AP_Dialog_Paragraph::spacing_SINGLE);
 	}
-	
+
 	{
 		// this block is our ACTIVE block
 		m_activeBlock = new AP_Preview_Paragraph_Block(*m_clrBlack,
@@ -449,13 +441,16 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_SPECIAL_INDENT),
 									(AP_Dialog_Paragraph::tIndentState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_SPECIAL_INDENT),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_LEFT_INDENT),
-									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_RIGHT_INDENT),								 
+									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_RIGHT_INDENT),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_BEFORE_SPACING),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_AFTER_SPACING),
 									dlg->_getSpinItemValue(AP_Dialog_Paragraph::id_SPIN_SPECIAL_SPACING),
 									(AP_Dialog_Paragraph::tSpacingState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_SPECIAL_SPACING));
+
+		if(dlg->_getCheckItemValue(AP_Dialog_Paragraph::id_CHECK_DOMDIRECTION) == AP_Dialog_Paragraph::check_TRUE)
+			m_dir = FRIBIDI_TYPE_RTL;
 	}
-	
+
 	{
 		// another dummy block
 		m_followingBlock = new AP_Preview_Paragraph_Block(*m_clrGray,
@@ -464,7 +459,7 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 														  m_fontHeight);
 		m_followingBlock->setFormat(dlg->m_pageLeftMargin,
 									dlg->m_pageRightMargin,
-									AP_Dialog_Paragraph::align_LEFT,
+									(AP_Dialog_Paragraph::tAlignState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_ALIGNMENT),
 									NULL,
 									AP_Dialog_Paragraph::indent_NONE,
 									NULL,NULL,NULL,NULL,NULL,
@@ -473,19 +468,12 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 
 	const XAP_StringSet * pSS = dlg->m_pApp->getStringSet();
 
-	
-	UT_UCSChar * tmp = NULL;
-
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewPrevParagraph));
-	m_previousBlock->setText(tmp);
-	FREEP(tmp);
+	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
 
-	UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewFollowParagraph));
-	m_followingBlock->setText(tmp);
-	FREEP(tmp);
+	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::~AP_Preview_Paragraph()
@@ -498,7 +486,7 @@ AP_Preview_Paragraph::~AP_Preview_Paragraph()
 
 	DELETEP(m_previousBlock);
 	DELETEP(m_activeBlock);
-	DELETEP(m_followingBlock);	
+	DELETEP(m_followingBlock);
 }
 
 void AP_Preview_Paragraph::setFormat(const XML_Char * pageLeftMargin,
@@ -535,7 +523,7 @@ void AP_Preview_Paragraph::draw(void)
 	_appendBlock(m_followingBlock);
 
 	_drawPageBorder();
-	
+
 	m_y = DEFAULT_TOP_MARGIN;
 }
 
@@ -543,7 +531,7 @@ bool AP_Preview_Paragraph::_loadDrawFont(void)
 {
 	// we draw at 7 points in this preview
 	GR_Font * font = m_gc->findFont("Times New Roman", "normal", "", "normal", "", "7pt");
-	
+
 	if (font)
 	{
 		REPLACEP(m_font, font);
@@ -558,17 +546,17 @@ bool AP_Preview_Paragraph::_loadDrawFont(void)
 void AP_Preview_Paragraph::_drawPageBackground(void)
 {
 	// clear area
-	m_gc->fillRect(*m_clrWhite, 0, 0, getWindowWidth(), getWindowHeight());
+	m_gc->fillRect(*m_clrWhite, 0, 0, m_gc->tlu(getWindowWidth()), m_gc->tlu(getWindowHeight()));
 }
 
 void AP_Preview_Paragraph::_drawPageBorder(void)
 {
 	// draw a black one pixel border
 	m_gc->setColor(*m_clrBlack);
-	m_gc->drawLine(0, 0, getWindowWidth(), 0);
-	m_gc->drawLine(getWindowWidth() - 1, 0, getWindowWidth() - 1, getWindowHeight());
-	m_gc->drawLine(getWindowWidth() - 1, getWindowHeight() - 1, 0, getWindowHeight() - 1);
-	m_gc->drawLine(0, getWindowHeight() - 1, 0, 0);
+	m_gc->drawLine(0, 0, m_gc->tlu(getWindowWidth()), 0);
+	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()));
+	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1));
+	m_gc->drawLine(0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, 0);
 }
 
 
@@ -578,10 +566,10 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 
 	UT_uint32 ypre = 0;
 	UT_uint32 ypost = 0;
-	
+
 	UT_uint32 wordCounter = 0;
 	UT_uint32 wordCount = block->m_words.getItemCount();
-		
+
 	m_gc->setColor(block->m_clr);
 
 	{
@@ -603,13 +591,13 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		}
 	}
-	
+
 	// start y at m_y;
 	UT_uint32 y = m_y;
 
 	// do before block spacing
 	y += block->m_beforeSpacing;
-	
+
 	// handle any spacing before first line
 	y += ypre;
 	// draw first line
@@ -623,7 +611,7 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 	y += block->m_fontHeight;
 	// handle any spacing after first line
 	y += ypost;
-	
+
 	// handle all other lines until out of words
 	UT_uint32 newWords = 1;
 	while (wordCounter < wordCount && newWords > 0)
@@ -648,7 +636,7 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 
 	// do after block spacing
 	y += block->m_afterSpacing;
-	
+
 	// record the changes
 	m_y = y;
 }
@@ -666,16 +654,16 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 	UT_ASSERT(words && widths);
 
 	// width of space character in pixels
-	UT_sint32 spaceCharWidth = 3;
+	UT_sint32 spaceCharWidth = m_gc->tlu(3);
 
 	UT_uint32 i = 0;
 	UT_uint32 totalWords = words->getItemCount();
-	
+
 	UT_sint32 pixelsForThisLine = 0;
-	
+
 	// max length of first line is the diff between the (special)
 	// left stop and the (normal) right stop
-	UT_sint32 maxPixelsForThisLine = getWindowWidth() - left - right;
+	UT_sint32 maxPixelsForThisLine = m_gc->tlu(getWindowWidth()) - left - right;
 
 	// negative or zero makes no sense; bail in that case (callers can deal)
 	if (maxPixelsForThisLine <= 0)
@@ -704,40 +692,79 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 	// we have "i" words to plot on this line, and they will take pixelsForThisLine space
 
 	// TODO : maybe rework following code to remove this variable for more speed
+
 	UT_uint32 willDrawAt = left;
-	spaceCharWidth <<= 8;	// Calculate spacing at 256 times the resolution
-		
+
+	if(m_dir == FRIBIDI_TYPE_RTL)
+		willDrawAt += maxPixelsForThisLine;
+
+ 	spaceCharWidth <<= 8;	// Calculate spacing at 256 times the resolution
+
 	// obey alignment requests
-	switch(align)
-	{
-	case AP_Dialog_Paragraph::align_RIGHT:
-		// for right, we just draw at the difference in spaces added onto the first line stop.
-		willDrawAt = left + (maxPixelsForThisLine - pixelsForThisLine);
+ 	switch(align)
+ 	{
+ 	case AP_Dialog_Paragraph::align_RIGHT:
+		if(m_dir == FRIBIDI_TYPE_LTR)
+
+	    // for right, we just draw at the difference in spaces added onto the first line stop.
+	    willDrawAt = left + (maxPixelsForThisLine - pixelsForThisLine);
 		break;
 	case AP_Dialog_Paragraph::align_CENTERED:
-		// for centered, we split the difference 
+		// for centered, we split the difference
 		willDrawAt = left + (maxPixelsForThisLine - pixelsForThisLine) / 2;
 		break;
 	case AP_Dialog_Paragraph::align_JUSTIFIED:
 		if(i < totalWords)
 		{
-			spaceCharWidth += (UT_sint32)((double)(maxPixelsForThisLine - pixelsForThisLine) / 
+			spaceCharWidth += (UT_sint32)((double)(maxPixelsForThisLine - pixelsForThisLine) /
 														(i - startWithWord) * 256);
 		}
 		break;
 	default:
 		// aligh_LEFT is caught here
+		if(m_dir == FRIBIDI_TYPE_RTL)
+			willDrawAt =  pixelsForThisLine + left;
+
 		break;
 	}
 
 	willDrawAt <<= 8;
 
 	UT_uint32 k;
+
+	FriBidiChar fb1[100];
+	FriBidiChar fb2[100];
+	UT_UCSChar  str[100];
+	UT_uint32 j, iLen;
+
 	for (k = startWithWord; k < i; k++)
 	{
-		m_gc->drawChars((UT_UCSChar *) words->getNthItem(k), 0,
-						UT_UCS_strlen((UT_UCSChar *) words->getNthItem(k)), willDrawAt >> 8, y);
-		willDrawAt += (((UT_uint32) widths->getNthItem(k)) << 8) + spaceCharWidth;
+		// this will not produce correct results in true bidi text, since the words that are inconsistend
+		// with the overall pargraph direction will be in wrong order, but that is not a big deal
+  	    iLen = UT_UCS4_strlen((UT_UCSChar *) words->getNthItem(k));
+	    for(j = 0; j < iLen; j++)
+		    fb1[j] = ((UT_UCSChar *) words->getNthItem(k))[j];
+
+		fribidi_log2vis (		/* input */
+						 &fb1[0],
+						 iLen,
+						 &m_dir,
+						 /* output */
+						 &fb2[0],
+						 NULL,
+						 NULL,
+						 NULL);
+
+		for(j = 0; j < iLen; j++)
+		    str[j] = (UT_UCSChar)fb2[j];
+
+		if(m_dir == FRIBIDI_TYPE_RTL)
+		    willDrawAt -= (((UT_uint32) widths->getNthItem(k)) << 8) + spaceCharWidth;
+
+		m_gc->drawChars(str, 0,	iLen, willDrawAt >> 8, y);
+
+		if(m_dir == FRIBIDI_TYPE_LTR)
+		    willDrawAt += (((UT_uint32) widths->getNthItem(k)) << 8) + spaceCharWidth;
 	}
 
 	// return number of words drawn
