@@ -347,6 +347,7 @@ dcl(DLG_Options_Label_ViewUnits,		"&Units:")
 dcl(DLG_Options_Label_ViewCursorBlink,	"Cursor &blink")
 dcl(DLG_Options_Label_ViewStandardTB,		"Standard Toolbar")
 dcl(DLG_Options_Label_ViewFormatTB, 	"Format Toolbar")
+dcl(DLG_Options_Label_ViewTableTB,		"Table Toolbar")
 dcl(DLG_Options_Label_ViewExtraTB,		"Extra Toolbar")
 dcl(DLG_Options_Label_ViewStatusBar,		"&Status bar")
 dcl(DLG_Options_Label_ViewViewFrame,	"View...")
@@ -790,10 +791,10 @@ dcl(MENU_LABEL_VIEW_NORMAL,"&Normal Layout")
 dcl(MENU_LABEL_VIEW_WEB,"&Web Layout")
 dcl(MENU_LABEL_VIEW_PRINT,"&Print Layout")
 dcl(MENU_LABEL_VIEW_TOOLBARS,"&Toolbars")
-dcl(MENU_LABEL_VIEW_TB_STD,"&Standard")
-dcl(MENU_LABEL_VIEW_TB_FORMAT,"&Formatting")
-dcl(MENU_LABEL_VIEW_TB_TABLE, "&Table")
-dcl(MENU_LABEL_VIEW_TB_EXTRA,"&Extra")
+dcl(MENU_LABEL_VIEW_TB_1,"&1 %s")
+dcl(MENU_LABEL_VIEW_TB_2,"&2 %s")
+dcl(MENU_LABEL_VIEW_TB_3,"&3 %s")
+dcl(MENU_LABEL_VIEW_TB_4,"&4 %s")
 dcl(MENU_LABEL_VIEW_LOCK_TB_LAYOUT,"&Lock layout")
 dcl(MENU_LABEL_VIEW_DEFAULT_TB_LAYOUT,"&Reset to default layout")
 dcl(MENU_LABEL_VIEW_RULER,"Show &Ruler")
@@ -1049,10 +1050,10 @@ dcl(MENU_STATUSLINE_VIEW_NORMAL,"Normal View")
 dcl(MENU_STATUSLINE_VIEW_WEB,"Web Layout")
 dcl(MENU_STATUSLINE_VIEW_PRINT,"Print Layout")
 dcl(MENU_STATUSLINE_VIEW_TOOLBARS," ")
-dcl(MENU_STATUSLINE_VIEW_TB_STD,"Show or hide the standard toolbar")
-dcl(MENU_STATUSLINE_VIEW_TB_FORMAT,"Show or hide the formatting toolbar")
-dcl(MENU_STATUSLINE_VIEW_TB_TABLE,"Show or hide the table toolbar")
-dcl(MENU_STATUSLINE_VIEW_TB_EXTRA,"Show or hide the extra toolbar")
+dcl(MENU_STATUSLINE_VIEW_TB_1,"Show or hide the toolbar")
+dcl(MENU_STATUSLINE_VIEW_TB_2,"Show or hide the toolbar")
+dcl(MENU_STATUSLINE_VIEW_TB_3,"Show or hide the toolbar")
+dcl(MENU_STATUSLINE_VIEW_TB_4,"Show or hide the toolbar")
 dcl(MENU_STATUSLINE_VIEW_LOCK_TB_LAYOUT,"Lock the layout of the current toolbars")
 dcl(MENU_STATUSLINE_VIEW_DEFAULT_TB_LAYOUT,"Reset the current toolbar layout to its defaults")
 dcl(MENU_STATUSLINE_VIEW_RULER,"Show or hide the rulers")
@@ -1258,6 +1259,11 @@ dcl(MENU_STATUSLINE_AUTOTEXT_EMAIL_5," ")
 dcl(MENU_STATUSLINE_AUTOTEXT_EMAIL_6," ")
 dcl(MENU_STATUSLINE_FMT_IMAGE, "Resize this image")
 
+/* Toolbar labels */
+dcl(TB_Extra,"Extra")
+dcl(TB_Format,"Format")
+dcl(TB_Standard,"Standard")
+dcl(TB_Table,"Table")
 
 dcl(TOOLBAR_LABEL_FILE_NEW,"New")
 dcl(TOOLBAR_LABEL_FILE_OPEN,"Open")
