@@ -9,6 +9,9 @@
 
 #define EXEC_ERROR 0x7FFFFFFF
 
+#ifdef NSIS_CONFIG_COMPONENTPAGE
+HWND g_lbSectionHack;
+#endif
 
 #ifdef NSIS_SUPPORT_STACK
 typedef struct _stack_t {
@@ -104,6 +107,8 @@ static LONG myRegDeleteKeyEx(HKEY thiskey, LPCTSTR lpSubKey, int onlyifempty)
 }
 #endif//NSIS_SUPPORT_REGISTRYFUNCTIONS
 
+extern char g_all_user_var_flag;
+
 static int ExecuteEntry(entry *entries, int pos);
 
 static int resolveaddr(int v) 
@@ -186,11 +191,13 @@ static int ExecuteEntry(entry *entries, int pos)
       {
         int x=process_string_fromtab_toint(parms[0]);
         if (x < 1) x=1;
-        update_status_text(LANG_STR(LANG_WAIT),"");
         log_printf2("Sleep(%d)",x);
         Sleep(x);
       }
     return 0;
+    case EW_SETSFCONTEXT:
+      g_all_user_var_flag=parms[0];
+    return 0;
     case EW_HIDEWINDOW:
       log_printf("HideWindow");
       ShowWindow(g_hwnd,SW_HIDE);
@@ -1300,7 +1307,72 @@ static int ExecuteEntry(entry *entries, int pos)
       }
     return 0;
 #endif//NSIS_CONFIG_UNINSTALL_SUPPORT
+#ifdef NSIS_CONFIG_LOG
+    case EW_LOG:
+      if (parms[0])
+      {
+        log_printf2("settings logging to %d",parms[1]);
+        log_dolog=parms[1];
+        log_printf2("logging set to %d",parms[1]);
+      }
+      else
+      {
+        process_string_fromtab(buf,parms[1]);
+        log_printf2("%s",buf);
+      }
+    return 0;
+#endif//NSIS_CONFIG_LOG
+#ifdef NSIS_CONFIG_COMPONENTPAGE
+    case EW_SECTIONSET:
+      {
+        int x=process_string_fromtab_toint(parms[0]);
+        if (g_inst_section && x >= 0 && x < g_inst_header->num_sections)
+        {
+          int z=0;
+          if (g_lbSectionHack)
+          {
+            int a;
+            for (a = 0; a < x; a ++) if (g_inst_section[a].name_ptr>=0) z++;
+          }
 
+          if (parms[1]==0) //set text
+          {
+            if (g_lbSectionHack)
+            {
+              if (g_inst_section[x].name_ptr>=0) 
+                SendMessage(g_lbSectionHack,LB_DELETESTRING,z,0);
+              
+              if (parms[2]>=0)
+              {
+                process_string_fromtab(buf,parms[2]);
+                SendMessage(g_lbSectionHack,LB_INSERTSTRING,z,(LPARAM)buf);
+                SendMessage(g_lbSectionHack,LB_SETITEMDATA,z,x);
+                SendMessage(g_lbSectionHack,LB_SETSEL,!!(g_inst_section[x].default_state&DFS_SET),z);
+              }
+            }
+            g_inst_section[x].name_ptr=parms[2];
+          }
+          else if (parms[1]==1) // get text
+          {
+            process_string_fromtab(g_usrvars[parms[2]],g_inst_section[x].name_ptr);
+          }
+          else if (parms[1]==2) // set flags
+          {
+            g_inst_section[x].default_state=process_string_fromtab_toint(parms[2]);
+            if (g_lbSectionHack && g_inst_section[x].name_ptr>=0)
+            {
+              SendMessage(g_lbSectionHack,LB_SETSEL,!!(g_inst_section[x].default_state&DFS_SET),z);
+            }
+          }
+          else // get flags
+          {
+            myitoa(g_usrvars[parms[2]],g_inst_section[x].default_state);
+          }
+        }
+        else exec_errorflag++;
+      }
+    return 0;
+#endif//NSIS_CONFIG_COMPONENTPAGE
   }
   MessageBox(g_hwnd,LANG_STR(LANG_INSTCORRUPTED),g_caption,MB_OK|MB_ICONSTOP);
   return EXEC_ERROR;
